@@ -227,9 +227,6 @@
 #define PLANTED_A_COMMAND 0x4EC
 #define NO_SUCH_FUNCTION  0x4EF
 
-/* A special keycode for when <Tab> is pressed while the mark is on. */
-#define INDENT_KEY  0x4F1
-
 /* A special keycode to signal the beginning and end of a bracketed paste. */
 #define BRACKETED_PASTE_MARKER  0x4FB
 
@@ -373,6 +370,7 @@ enum {
 	EMPTY_LINE,
 	INDICATOR,
 	BOOKSTYLE,
+	COLON_PARSING,
 	STATEFLAGS,
 	USE_MAGIC,
 	MINIBAR,
