@@ -1,7 +1,7 @@
 /**************************************************************************
  *   rcfile.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 2001-2011, 2013-2023 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2001-2011, 2013-2024 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mike Frysinger                                    *
  *   Copyright (C) 2019 Brand Huntsman                                    *
  *   Copyright (C) 2014-2021 Benno Schulenberg                            *
@@ -69,7 +69,7 @@ static const rcoption rcopts[] = {
 	{"nohelp", NO_HELP},
 	{"nonewlines", NO_NEWLINES},
 #ifdef ENABLE_WRAPPING
-	{"nowrap", NO_WRAP},  /* Deprecated; remove in 2024. */
+	{"nowrap", NO_WRAP},  /* Deprecated; remove in 2027. */
 #endif
 #ifdef ENABLE_OPERATINGDIR
 	{"operatingdir", 0},
@@ -98,6 +98,7 @@ static const rcoption rcopts[] = {
 	{"backup", MAKE_BACKUP},
 	{"backupdir", 0},
 	{"bookstyle", BOOKSTYLE},
+	{"colonparsing", COLON_PARSING},
 	{"cutfromcursor", CUT_FROM_CURSOR},
 	{"emptyline", EMPTY_LINE},
 	{"guidestripe", 0},
@@ -245,6 +246,8 @@ keystruct *strtosc(const char *input)
 		s->func = do_replace;
 	else if (!strcmp(input, "cut"))
 		s->func = cut_text;
+	else if (!strcmp(input, "copy"))
+		s->func = copy_text;
 	else if (!strcmp(input, "paste"))
 		s->func = paste_text;
 #ifndef NANO_TINY
@@ -252,8 +255,6 @@ keystruct *strtosc(const char *input)
 		s->func = do_execute;
 	else if (!strcmp(input, "cutrestoffile"))
 		s->func = cut_till_eof;
-	else if (!strcmp(input, "copy"))
-		s->func = copy_text;
 	else if (!strcmp(input, "zap"))
 		s->func = zap_text;
 	else if (!strcmp(input, "mark"))
@@ -321,6 +322,8 @@ keystruct *strtosc(const char *input)
 		s->func = do_undo;
 	else if (!strcmp(input, "redo"))
 		s->func = do_redo;
+	else if (!strcmp(input, "suspend"))
+		s->func = do_suspend;
 #endif
 	else if (!strcmp(input, "left") ||
 	         !strcmp(input, "back"))
@@ -339,8 +342,6 @@ keystruct *strtosc(const char *input)
 		s->func = do_scroll_up;
 	else if (!strcmp(input, "scrolldown"))
 		s->func = do_scroll_down;
-	else if (!strcmp(input, "center"))
-		s->func = do_center;
 #endif
 	else if (!strcmp(input, "prevword"))
 		s->func = to_prev_word;
@@ -354,6 +355,16 @@ keystruct *strtosc(const char *input)
 		s->func = to_prev_block;
 	else if (!strcmp(input, "nextblock"))
 		s->func = to_next_block;
+#ifndef NANO_TINY
+	else if (!strcmp(input, "toprow"))
+		s->func = to_top_row;
+	else if (!strcmp(input, "bottomrow"))
+		s->func = to_bottom_row;
+	else if (!strcmp(input, "center"))
+		s->func = do_center;
+	else if (!strcmp(input, "cycle"))
+		s->func = do_cycle;
+#endif
 	else if (!strcmp(input, "pageup") ||
 	         !strcmp(input, "prevpage"))
 		s->func = do_page_up;
@@ -382,10 +393,6 @@ keystruct *strtosc(const char *input)
 		s->func = do_backspace;
 	else if (!strcmp(input, "refresh"))
 		s->func = full_refresh;
-#ifndef NANO_TINY
-	else if (!strcmp(input, "suspend"))
-		s->func = do_suspend;
-#endif
 	else if (!strcmp(input, "casesens"))
 		s->func = case_sens_void;
 	else if (!strcmp(input, "regexp"))
@@ -463,8 +470,7 @@ keystruct *strtosc(const char *input)
 		else if (!strcmp(input, "cutfromcursor"))
 			s->toggle = CUT_FROM_CURSOR;
 #ifdef ENABLE_WRAPPING
-		else if (!strcmp(input, "breaklonglines") ||
-		         !strcmp(input, "nowrap"))  /* Deprecated; remove in 2024. */
+		else if (!strcmp(input, "breaklonglines"))
 			s->toggle = BREAK_LONG_LINES;
 #endif
 		else if (!strcmp(input, "tabstospaces"))
@@ -674,12 +680,12 @@ void begin_new_syntax(char *ptr)
 	live_syntax->magics = NULL;
 	live_syntax->linter = NULL;
 	live_syntax->formatter = NULL;
-	live_syntax->tab = NULL;
+	live_syntax->tabstring = NULL;
 #ifdef ENABLE_COMMENT
 	live_syntax->comment = copy_of(GENERAL_COMMENT_CHARACTER);
 #endif
 	live_syntax->color = NULL;
-	live_syntax->nmultis = 0;
+	live_syntax->multiscore = 0;
 
 	/* Hook the new syntax in at the top of the list. */
 	live_syntax->next = syntaxes;
@@ -974,6 +980,11 @@ void parse_includes(char *ptr)
 		pattern++;
 	ptr = parse_argument(ptr);
 
+	if (strlen(pattern) > PATH_MAX) {
+		jot_error(N_("Path is too long"));
+		return;
+	}
+
 	/* Expand a tilde first, then try to match the globbing pattern. */
 	expanded = real_dir_from_tilde(pattern);
 	result = glob(expanded, GLOB_ERR|GLOB_NOCHECK, NULL, &files);
@@ -991,16 +1002,22 @@ void parse_includes(char *ptr)
 }
 
 /* Return the index of the color that is closest to the given RGB levels,
- * assuming that the terminal uses the 6x6x6 color cube of xterm-256color. */
+ * assuming that the terminal uses the 6x6x6 color cube of xterm-256color.
+ * When red == green == blue, return an index in the xterm gray scale. */
 short closest_index_color(short red, short green, short blue)
 {
-	/* Translation table, from 16 intended levels to 6 available levels. */
+	/* Translation table, from 16 intended color levels to 6 available levels. */
 	static const short level[] = { 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5 };
 
-	if (COLORS == 256)
-		return (36 * level[red] + 6 * level[green] + level[blue] + 16);
-	else
+	/* Translation table, from 14 intended gray levels to 24 available levels. */
+	static const short gray[] = { 1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, 18, 21, 23 };
+
+	if (COLORS != 256)
 		return THE_DEFAULT;
+	else if (red == green && green == blue && 0 < red && red < 0xF)
+		return 232 + gray[red - 1];
+	else
+		return (36 * level[red] + 6 * level[green] + level[blue] + 16);
 }
 
 #define COLORCOUNT  34
@@ -1031,7 +1048,7 @@ short indices[COLORCOUNT] = { COLOR_RED, COLOR_GREEN, COLOR_BLUE,
 short color_to_short(const char *colorname, bool *vivid, bool *thick)
 {
 	if (strncmp(colorname, "bright", 6) == 0 && colorname[6] != '\0') {
-		/* Prefix "bright" is deprecated; remove in 2024. */
+		/* Prefix "bright" is deprecated; remove in 2027. */
 		*vivid = TRUE;
 		*thick = TRUE;
 		colorname += 6;
@@ -1073,40 +1090,40 @@ short color_to_short(const char *colorname, bool *vivid, bool *thick)
 
 /* Parse the color name (or pair of color names) in the given string.
  * Return FALSE when any color name is invalid; otherwise return TRUE. */
-bool parse_combination(char *combostr, short *fg, short *bg, int *attributes)
+bool parse_combination(char *combotext, short *fg, short *bg, int *attributes)
 {
 	bool vivid, thick;
 	char *comma;
 
 	*attributes = A_NORMAL;
 
-	if (strncmp(combostr, "bold", 4) == 0) {
+	if (strncmp(combotext, "bold", 4) == 0) {
 		*attributes |= A_BOLD;
-		if (combostr[4] != ',') {
+		if (combotext[4] != ',') {
 			jot_error(N_("An attribute requires a subsequent comma"));
 			return FALSE;
 		}
-		combostr += 5;
+		combotext += 5;
 	}
 
-	if (strncmp(combostr, "italic", 6) == 0) {
+	if (strncmp(combotext, "italic", 6) == 0) {
 #ifdef A_ITALIC
 		*attributes |= A_ITALIC;
 #endif
-		if (combostr[6] != ',') {
+		if (combotext[6] != ',') {
 			jot_error(N_("An attribute requires a subsequent comma"));
 			return FALSE;
 		}
-		combostr += 7;
+		combotext += 7;
 	}
 
-	comma = strchr(combostr, ',');
+	comma = strchr(combotext, ',');
 
 	if (comma)
 		*comma = '\0';
 
-	if (!comma || comma > combostr) {
-		*fg = color_to_short(combostr, &vivid, &thick);
+	if (!comma || comma > combotext) {
+		*fg = color_to_short(combotext, &vivid, &thick);
 		if (*fg == BAD_COLOR)
 			return FALSE;
 		if (vivid && !thick && COLORS > 8)
@@ -1212,22 +1229,22 @@ void parse_rule(char *ptr, int rex_flags)
 
 		/* For a multiline rule, give it a number and increase the count. */
 		if (expectend) {
-			newcolor->id = live_syntax->nmultis;
-			live_syntax->nmultis++;
+			newcolor->id = live_syntax->multiscore;
+			live_syntax->multiscore++;
 		}
 	}
 }
 
-/* Parse the argument of an interface color option. */
-colortype *parse_interface_color(char *combostr)
+/* Set the colors for the given interface element to the given combination. */
+void set_interface_color(int element, char *combotext)
 {
 	colortype *trio = nmalloc(sizeof(colortype));
 
-	if (!parse_combination(combostr, &trio->fg, &trio->bg, &trio->attributes)) {
-		free(trio);
-		return NULL;
+	if (parse_combination(combotext, &trio->fg, &trio->bg, &trio->attributes)) {
+		free(color_combo[element]);
+		color_combo[element] = trio;
 	} else
-		return trio;
+		free(trio);
 }
 
 /* Read regex strings enclosed in double quotes from the line pointed at
@@ -1325,12 +1342,14 @@ bool parse_syntax_commands(char *keyword, char *ptr)
 		pick_up_name("comment", ptr, &live_syntax->comment);
 #endif
 	} else if (strcmp(keyword, "tabgives") == 0) {
-		pick_up_name("tabgives", ptr, &live_syntax->tab);
-	} else if (strcmp(keyword, "linter") == 0)
+		pick_up_name("tabgives", ptr, &live_syntax->tabstring);
+	} else if (strcmp(keyword, "linter") == 0) {
 		pick_up_name("linter", ptr, &live_syntax->linter);
-	else if (strcmp(keyword, "formatter") == 0)
+		strip_leading_blanks_from(live_syntax->linter);
+	} else if (strcmp(keyword, "formatter") == 0) {
 		pick_up_name("formatter", ptr, &live_syntax->formatter);
-	else
+		strip_leading_blanks_from(live_syntax->formatter);
+	} else
 		return FALSE;
 
 	return TRUE;
@@ -1347,7 +1366,7 @@ static void check_vitals_mapped(void)
 
 	for (int v = 0; v < VITALS; v++) {
 		for (funcstruct *f = allfuncs; f != NULL; f = f->next) {
-			if (f->func == vitals[v] && f->menus & inmenus[v]) {
+			if (f->func == vitals[v] && (f->menus & inmenus[v])) {
 				if (first_sc_for(inmenus[v], f->func) == NULL) {
 					jot_error(N_("No key is bound to function '%s' in menu '%s'. "
 								" Exiting.\n"), f->tag, menu_to_name(inmenus[v]));
@@ -1570,29 +1589,29 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 #endif
 #ifdef ENABLE_COLOR
 		if (strcmp(option, "titlecolor") == 0)
-			color_combo[TITLE_BAR] = parse_interface_color(argument);
+			set_interface_color(TITLE_BAR, argument);
 		else if (strcmp(option, "numbercolor") == 0)
-			color_combo[LINE_NUMBER] = parse_interface_color(argument);
+			set_interface_color(LINE_NUMBER, argument);
 		else if (strcmp(option, "stripecolor") == 0)
-			color_combo[GUIDE_STRIPE] = parse_interface_color(argument);
+			set_interface_color(GUIDE_STRIPE, argument);
 		else if (strcmp(option, "scrollercolor") == 0)
-			color_combo[SCROLL_BAR] = parse_interface_color(argument);
+			set_interface_color(SCROLL_BAR, argument);
 		else if (strcmp(option, "selectedcolor") == 0)
-			color_combo[SELECTED_TEXT] = parse_interface_color(argument);
+			set_interface_color(SELECTED_TEXT, argument);
 		else if (strcmp(option, "spotlightcolor") == 0)
-			color_combo[SPOTLIGHTED] = parse_interface_color(argument);
+			set_interface_color(SPOTLIGHTED, argument);
 		else if (strcmp(option, "minicolor") == 0)
-			color_combo[MINI_INFOBAR] = parse_interface_color(argument);
+			set_interface_color(MINI_INFOBAR, argument);
 		else if (strcmp(option, "promptcolor") == 0)
-			color_combo[PROMPT_BAR] = parse_interface_color(argument);
+			set_interface_color(PROMPT_BAR, argument);
 		else if (strcmp(option, "statuscolor") == 0)
-			color_combo[STATUS_BAR] = parse_interface_color(argument);
+			set_interface_color(STATUS_BAR, argument);
 		else if (strcmp(option, "errorcolor") == 0)
-			color_combo[ERROR_MESSAGE] = parse_interface_color(argument);
+			set_interface_color(ERROR_MESSAGE, argument);
 		else if (strcmp(option, "keycolor") == 0)
-			color_combo[KEY_COMBO] = parse_interface_color(argument);
+			set_interface_color(KEY_COMBO, argument);
 		else if (strcmp(option, "functioncolor") == 0)
-			color_combo[FUNCTION_TAG] = parse_interface_color(argument);
+			set_interface_color(FUNCTION_TAG, argument);
 		else
 #endif
 #ifdef ENABLE_OPERATINGDIR
